<?php
namespace FORM4\Form4Teaser\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script
 ***************************************************************/

use \FORM4\Form4Teaser\Utility\ConfigurationUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 * 
 * @package TYPO3
 * @subpackage form4_teaser
 */
class TeaserController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * @var \FORM4\Form4Teaser\Domain\Repository\LargePageRepository
	 * @inject
	 */
	protected $pageRepository;

	/**
	 * The default view object to use if none of the resolved views can render
	 * a response for the current request.
	 *
	 * @var string
	 * @api
	 */
	protected $defaultViewObjectName = 'FORM4\\Form4Teaser\\View\\TeaserView';


	/**
	 * Override flexform fields with typoscript settings when they are disabled via pageTSconfig.
	 *
	 * @see \TYPO3\CMS\Extbase\Mvc\Controller\ActionController::initializeAction()
	 */
	public function initializeAction() {
		$fullTs = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);
		$tsSettings = $fullTs['plugin.']['tx_form4teaser.']['settings.'];
		$flexformSettings = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);

		$pageTSconfig = $GLOBALS['TSFE']->getPagesTSconfig();
		$pluginPageTSconfig = $pageTSconfig['TCEFORM.']['tt_content.']['pi_flexform.']['form4teaser_teaser.'];

		$this->settings = $flexformSettings;

		foreach ($tsSettings as $key => $value) {
			foreach ($pluginPageTSconfig as $sheet) {
				$configKey = 'settings.' . $key . '.';
				if (isset($sheet[$configKey]) && isset($sheet[$configKey]['disabled']) && $sheet[$configKey]['disabled'] == 1) {
					$this->settings[$key] = $tsSettings[$key];
					break;
				}
			}
		}
	}

	/**
	 * action index
	 *
	 * @return void
	 */
	public function indexAction() {
		// get the actual array of allowed doktypes
		$this->settings['filterDoktypes'] = $this->getAllowedDoktypes();
		
		// get pages with the flexform settings
		$pages = $this->pageRepository->getPagesWithFlexformContraints($this->settings);
		$paginationactive = $this->settings['pagingVisibility'];

		// prepare pagination information for the layout
		if ($paginationactive) {
			switch ($this->settings['pagingPosition']) {
				case 'top':
					$paginationtop = 1;
					$paginationbottom = 0;
				break;
				case 'bottom':
					$paginationtop = 0;
					$paginationbottom = 1;
				break;
				case 'topAndBottom':
					$paginationtop = 1;
					$paginationbottom = 1;
				break;
				default:
					$paginationactive = 0;
					$paginationbottom = 0;
				break;
			}
		}
		

		//needed for adjusted teaser layout!
		$this->view->assign('pages', $pages);

		$this->view->assign('paginationactive', $paginationactive);
		$this->view->assign('paginationtop', $paginationtop);
		$this->view->assign('paginationbottom', $paginationbottom);
		
		// assign the data array of the current content object to the view 
		$this->view->assign('cObjectData', $this->configurationManager->getContentObject()->data);
	}
	
	/**
	 * Get an array of allowed Doktypes
	 * The list is either the one maintained in the plugin settings
	 * or if it's empty the whitelist minus the blacklist from the extConf.
	 * 
	 * @return array
	 */
	protected function getAllowedDoktypes() {
		$doktypes = GeneralUtility::intExplode(',', $this->settings['filterDoktypes'], TRUE);
		$extensionConfiguration = unserialize($GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['form4_teaser']);
		
		if (empty($doktypes)) {
			$doktypes = GeneralUtility::intExplode(',', $extensionConfiguration['allowedDoktypes'], TRUE);
		}
		
		$forbiddenDoktypes = GeneralUtility::intExplode(',', $extensionConfiguration['forbiddenDoktypes'], TRUE);
		$doktypes = array_filter($doktypes, function($element) use ($forbiddenDoktypes) {
			return !in_array($element, $forbiddenDoktypes);
		});
		
		return $doktypes;
	}

}
?>