<?php
namespace FORM4\Form4Teaser\Utility;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Backend\Utility\BackendUtility;
use \FORM4\Form4Teaser\Utility\ConfigurationUtility;

/**
 * This class provides methods for getting lists of available plugin configuration options as HTML Select boxes.
 * It's intended to be used as user defined TS constant types.
 * 
 * @package TYPO3
 * @subpackage form4_teaser
 *
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 */
class ConstantUserTypesUtility implements \TYPO3\CMS\Core\SingletonInterface {

	public function getSources ($params, $pObj) {
		$items = $this->getItemsFromFlexform('settings.source', 'sDEF');
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], $items);
	}

	public function getSortings ($params, $pObj) {
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], ConfigurationUtility::getSorting());
	}

	public function getSortingDirections ($params, $pObj) {
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], ConfigurationUtility::getSortingDirection());
	}

	public function getLayouts ($params, $pObj) {
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], ConfigurationUtility::getLayout());
	}

	public function getPagingVisibilites ($params, $pObj) {
		$items = $this->getItemsFromFlexform('settings.pagingVisibility', 'layoutsheet');
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], $items);
	}

	public function getPagingPositions ($params, $pObj) {
		$items = $this->getItemsFromFlexform('settings.pagingPosition', 'layoutsheet');
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], $items);
	}

	public function getRecursive ($params, $pObj) {
		$items = $this->getItemsFromFlexform('settings.recursive', 'sDEF');
		return $this->createSelectElementFromConfigurationArray($params['fieldName'], $params['fieldValue'], $items);
	}

	/**
	 * Reads a given setting of the teaser plugin flexform and returns its items.
	 * 
	 * @param string $field
	 * @param string $sheet
	 * @return array Associative array of associative arrays ['option1' => ['label' => 'string'], 'option2' => ['label' => 'string'], ...]
	 */
	protected function getItemsFromFlexform ($field, $sheet) {
		$registeredFlexforms = BackendUtility::getRegisteredFlexForms('tt_content');
		$items = array();
		foreach ($registeredFlexforms['pi_flexform']['form4teaser_teaser']['ds']['sheets'][$sheet]['ROOT']['el'][$field]['TCEforms']['config']['items'] as $item) {
			list($label, $key) = $item;
			$items[$key] = array('label' => $label);
		}
		return $items;
	}

	/**
	 * Renders a HTML select box.
	 * 
	 * @param string $fieldName
	 * @param mixed $fieldValue
	 * @param array $array
	 * @return string
	 */
	protected function createSelectElementFromConfigurationArray ($fieldName, $fieldValue, array $array) {
		$tag = '<select name="' . $fieldName . '">';

		foreach ($array as $key => $data) {
			$selected = ((string) $fieldValue === (string) $key)
				? ' selected="selected"'
				: '';
			$tag .= '<option value="' . $key . '"' . $selected . '>' . $GLOBALS['LANG']->sL($data['label']) . '</option>';
		}

		$tag .= '</select>';

		return $tag;
	}



}