<?php
namespace FORM4\Form4Teaser\ViewHelpers\Widget;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 form4 GmbH & Co. KG <typo3@form4.de>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script
 ***************************************************************/


use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * This ViewHelper extends the original LinkViewHelper by the ability to distinguish between different paginated plugins on the same page.
 *
 * @package TYPO3
 * @subpackage form4_teaser
 *
 * @author Thomas Grothaus <thomas.grothaus@form4.de>
 * @author Stefan Szymanski <stefan.szymanski@form4.de>
 */
class LinkViewHelper extends \TYPO3\CMS\Fluid\ViewHelpers\Widget\LinkViewHelper {

	/**
	 * @var integer
	 */
	protected $contentId;

	/**
	 * @var \FORM4\Form4Teaser\ViewHelpers\Widget\Controller\LinkController
	 * @inject
	 */
	protected $controller;


	/**
	 * Render the link.
	 *
	 * @param string $action Target action
	 * @param array $arguments Arguments
	 * @param string $section The anchor to be added to the URI
	 * @param string $format The requested format, e.g. ".html
	 * @param boolean $ajax TRUE if the URI should be to an AJAX widget, FALSE otherwise.
	 * @return string The rendered link
	 * @api
	 */
	public function render($action = NULL, $arguments = array(), $section = '', $format = '', $ajax = FALSE) {
		$this->contentId = (int) $this->controller->getContentUid();
		if (empty($this->contentId)) {
			return '<span class="linkviewhelper-error">' . \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('viewhelper.link.missingcontentid', 'form4_teaser') . '</span>';
		}

		$uri = $ajax
			? $this->getAjaxUri()
			: $this->getWidgetUri();

		$this->tag->addAttribute('href', $uri);
		$this->tag->setContent($this->renderChildren());

		return $this->tag->render();
	}

	/**
	 * Get the URI for a non-AJAX Request.
	 * Preserve pagination information of other teaser plugins on the same page.
	 *
	 * @return string the Widget URI
	 */
	protected function getWidgetUri() {
		$arguments = $this->hasArgument('arguments') ? $this->arguments['arguments'] : array();


		// make a nested array from the currentPage string
		// when the current plugin is already in the list its page value is updated
		$allArguments = $this->controllerContext->getRequest()->getArguments();
		$currentPages = GeneralUtility::trimExplode(';', $allArguments['currentPage']);
		foreach ($currentPages as & $currentPage) {
			list($cid, $page) = GeneralUtility::trimExplode(':', $currentPage);
			if ((int) $cid === $this->contentId) {
				$page = $arguments['currentPage'];
				$currentPage = "$cid:$page";
				break;
			}
		}
		// $currentPages had no pagination info about $this->contentId. So we add it here
		if ((int) $cid !== $this->contentId) {
			$page = $arguments['currentPage'];
			$currentPages[] = "$this->contentId:$page";
		}
		// implode back to a string
		$currentPages = implode(';', $currentPages);

		$arguments['currentPage'] = $currentPages;

		if ($this->hasArgument('action')) {
			$arguments['action'] = $this->arguments['action'];
		}
		if ($this->hasArgument('format') && $this->arguments['format'] !== '') {
			$arguments['format'] = $this->arguments['format'];
		}
		
		$uriBuilder = $this->controllerContext->getUriBuilder();
		$argumentPrefix = $this->controllerContext->getRequest()->getArgumentPrefix();
		return $uriBuilder
			->reset()
			->setArguments(array($argumentPrefix => $arguments))
			->setSection($this->arguments['section'])
			->setAddQueryString(TRUE)
			->setArgumentsToBeExcludedFromQueryString(array($argumentPrefix, 'cHash'))
			->setFormat($this->arguments['format'])
			->build();
	}

}

?>